package com.sxtvs.api.chatgpt.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.api.chatgpt.dto.*;
import com.sxtvs.core.sls.AliyunLogger;
import com.sxtvs.core.sls.advice.BizException;
import lombok.Cleanup;
import lombok.SneakyThrows;
import org.apache.commons.io.IOUtils;
import org.apache.hc.core5.http.HttpStatus;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestBody;

import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;

@Service
public class ChatGptService {

    @Autowired
    private AliyunLogger logger;

    @Autowired
    private ObjectMapper objectMapper;

    private final CloseableHttpClient client = HttpClients.createDefault();

    private final ArrayBlockingQueue<String> queue = new ArrayBlockingQueue<>(1);
    {
        try {
//            queue.put("Bearer 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");
            queue.put("Bearer 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");
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }


    @SneakyThrows
    public CompletionsResponseDto completions(@RequestBody CompletionsRequestDto dto) {
        var paramsDto = new CompletionsParamsDto(dto.getPrompt()
                .stream()
                .map(CompletionsRequestDto.Prompt::getText)
                .collect(Collectors.joining("\n")));
        var params = objectMapper.writeValueAsString(paramsDto);
        logger.info("request", params, "dto", objectMapper.writeValueAsString(dto));
        var token = queue.poll();
        String result;
        try {
            var httpPost = new HttpPost("https://api.openai.com/v1/completions");
            httpPost.setHeader("Authorization", token);
            httpPost.setEntity(new StringEntity(params, ContentType.APPLICATION_JSON));
            @Cleanup
            var response = client.execute(httpPost);
            @Cleanup
            var content = response.getEntity().getContent();
            result = IOUtils.toString(content, StandardCharsets.UTF_8);
            logger.info("response", result);
            if (response.getStatusLine().getStatusCode() >= HttpStatus.SC_REDIRECTION) {
                logger.error("error", result);
                throw new BizException(1000, "服务器走丢了 请重试");
            }
        } finally {
            queue.put(token);
        }

        var gptResponse = objectMapper.readValue(result, GptResponse.class);

        var completionsResponseDto = new CompletionsResponseDto();
        var text = gptResponse.getChoices().stream().map(GptResponse.ChoicesDTO::getText).findFirst().orElse("").trim();
        completionsResponseDto.setResult(text);
        return completionsResponseDto;
    }
}
